@@ -32,7 +32,7 @@ require_once 'include/version.php';
 
 function cmpReps($a, $b) {
 	// First, sort by group
-	$g = strcasecmp($a->group, $b->group);
+	$g = strcasecmp((string) $a->group, (string) $b->group);
 	if ($g) return $g;
 
 	// Same group? Sort by name
@@ -44,7 +44,7 @@ function cmpReps($a, $b) {
 // {{{ cmpGroups($a, $b)
 
 function cmpGroups($a, $b) {
-	$g = strcasecmp($a->group, $b->group);
+	$g = strcasecmp((string) $a->group, (string) $b->group);
 	if ($g) return $g;
 
 	return 0;
@@ -60,7 +60,7 @@ function mergesort(&$array, $cmp_function = 'strcmp') {
 	if (count($array) < 2) return;
 
 	// Split the array in half
-	$halfway = count($array) / 2;
+	$halfway = ((int) (count($array) / 2));
 	$array1 = array_slice($array, 0, $halfway);
 	$array2 = array_slice($array, $halfway);
 
@@ -143,7 +143,7 @@ class ParentPath {
 				if ($this->pattern === false || preg_match($this->pattern, $name)) {
 					$url = $config->fileUrlPrefix.$fullpath;
 					$url = str_replace(DIRECTORY_SEPARATOR, '/', $url);
-					if ($url{strlen($url) - 1} == '/') {
+					if ($url[ strlen($url) - 1 ] == '/') {
 						$url = substr($url, 0, -1);
 					}
 
@@ -172,7 +172,7 @@ class ParentPath {
 		// For each file...
 		while (false !== ($name = readdir($handle))) {
 			$fullpath = $this->path.DIRECTORY_SEPARATOR.$name;
-			if ($name{0} != '.' && is_dir($fullpath) && is_readable($fullpath)) {
+			if ($name[0] != '.' && is_dir($fullpath) && is_readable($fullpath)) {
 				// And that contains a db directory (in an attempt to not include non svn repositories.
 				$dbfullpath = $fullpath.DIRECTORY_SEPARATOR.'db';
 				if (is_dir($dbfullpath) && is_readable($dbfullpath)) {
@@ -180,7 +180,7 @@ class ParentPath {
 					if ($this->pattern === false || preg_match($this->pattern, $name)) {
 						$url = $config->fileUrlPrefix.$fullpath;
 						$url = str_replace(DIRECTORY_SEPARATOR, '/', $url);
-						if ($url{strlen($url) - 1} == '/') {
+						if ($url[strlen($url) - 1] == '/') {
 							$url = substr($url, 0, -1);
 						}
 						$clientRootURL = ($this->clientRootURL) ? $this->clientRootURL.'/'.$name : '';
@@ -377,13 +377,13 @@ class Repository {
 	}
 
 	function addAllowedDownloadException($path) {
-		if ($path{strlen($path) - 1} != '/') $path .= '/';
+		if ($path[strlen($path) - 1] != '/') $path .= '/';
 
 		$this->allowedExceptions[] = $path;
 	}
 
 	function addDisallowedDownloadException($path) {
-		if ($path{strlen($path) - 1} != '/') $path .= '/';
+		if ($path[strlen($path) - 1] != '/') $path .= '/';
 
 		$this->disallowedExceptions[] = $path;
 	}
@@ -584,7 +584,8 @@ class Repository {
 	}
 
 	function hasLogReadAccess($pathWithSub) {
-                $a =& $this->getAuthz();
+		$path	=  $pathWithSub;
+		$a		=& $this->getAuthz();
 
 		if (!empty($a)) {
 			return $a->hasReadAccess($this->svnName, $path, false);
@@ -617,7 +618,6 @@ class WebSvnConfig {
 
 	// Tool path locations
 
-	var $_svnCommandPrefix = '';
 	var $_svnCommandPath = '';
 	var $_svnConfigDir = '/tmp/websvn';
 	var $_svnTrustServerCert = false;
@@ -630,6 +630,7 @@ class WebSvnConfig {
 	var $tar = 'tar';
 	var $zip = 'zip';
 	var $locale = '';
+	var $characterEncodings = array( 'UTF-8', 'ISO-8859-1', 'windows-1252' );
 
 	// different modes for file and directory download
 
@@ -656,6 +657,9 @@ class WebSvnConfig {
 	var $useEnscript = false;
 	var $useEnscriptBefore_1_6_3 = false;
 	var $useGeshi = false;
+	var $geshiScript = 'geshi.php';
+	var $useParsedown = false;
+	var $parsedownScript = 'Parsedown.php';
 	var $inlineMimeTypes = array();
 	var $allowDownload = false;
 	var $tempDir = '';
@@ -672,6 +676,8 @@ class WebSvnConfig {
 	var $authz = null;
 	var $blockRobots = false;
 
+	var $loadAllRepos = false;
+
 	var $templatePaths = array();
 	var $userTemplate = false;
 
@@ -688,6 +694,7 @@ class WebSvnConfig {
 	var $quote = "'";
 	var $pathSeparator = ':';
 	var $fileUrlPrefix = 'file://';
+	var $breadcrumbRepoRootAsRepo = false;
 
 	var $_repositories = array();
 
@@ -733,7 +740,7 @@ class WebSvnConfig {
 	function addExcludedPath($path) {
 		$url = $this->fileUrlPrefix.$path;
 		$url = str_replace(DIRECTORY_SEPARATOR, '/', $url);
-		if ($url{strlen($url) - 1} == '/') {
+		if ($url[strlen($url) - 1] == '/') {
 			$url = substr($url, 0, -1);
 		}
 		$this->_excluded[] = $url;
@@ -861,6 +868,14 @@ class WebSvnConfig {
 
 	// {{{ GeSHi
 
+	function setGeshiPath($path) {
+		$this->_setPath($this->geshiScript, $path, 'geshi.php');
+	}
+
+	function getGeshiScript() {
+		return $this->geshiScript;
+	}
+
 	// useGeshi
 	//
 	// Use GeSHi to colourise listings
@@ -874,6 +889,29 @@ class WebSvnConfig {
 
 	// }}}
 
+	// {{{ Parsedown
+
+	function setParsedownPath($path) {
+		$this->_setPath($this->parsedownScript, $path, 'Parsedown.php');
+	}
+
+	function getParsedownScript() {
+		return $this->parsedownScript;
+	}
+
+	// useParsedown
+	//
+	// Use Parsedown to render README.md
+	function useParsedown() {
+		$this->useParsedown = true;
+	}
+
+	function getUseParsedown() {
+		return $this->useParsedown;
+	}
+
+	// }}}
+
 	// {{{ Inline MIME Types
 
 	// inlineMimeTypes
@@ -1004,21 +1042,6 @@ class WebSvnConfig {
 
 	function getTempDir() {
 		if (empty($this->tempDir)) {
-			if (!function_exists('sys_get_temp_dir')) {
-				function sys_get_temp_dir() {
-					if (($tmp = getenv('TMPDIR')) ||
-						($tmp = getenv('TMP')) ||
-						($tmp = getenv('TEMP')) ||
-						($tmp = ini_get('upload_tmp_dir')))
-						return $tmp;
-					$tmp = tempnam(__FILE__, '');
-					if (file_exists($tmp)) {
-						unlink($tmp);
-						return dirname($tmp);
-					}
-					return null;
-				}
-			}
 			$this->tempDir = sys_get_temp_dir();
 		}
 		return $this->tempDir;
@@ -1038,7 +1061,7 @@ class WebSvnConfig {
 	}
 
 	function addAllowedDownloadException($path, $myrep = 0) {
-		if ($path{strlen($path) - 1} != '/') {
+		if ($path[strlen($path) - 1] != '/') {
 			$path .= '/';
 		}
 
@@ -1051,7 +1074,7 @@ class WebSvnConfig {
 	}
 
 	function addDisallowedDownloadException($path, $myrep = 0) {
-		if ($path{strlen($path) - 1} != '/') {
+		if ($path[strlen($path) - 1] != '/') {
 			$path .= '/';
 		}
 
@@ -1083,7 +1106,7 @@ class WebSvnConfig {
 		list($base, $params) = $this->getUrlParts($rep, $path, $op);
 		$url = $base.'?';
 		foreach ($params as $k => $v) {
-			$url .= $k.'='.urlencode($v).'&amp;';
+			$url .= $k.'='.rawurlencode($v).'&amp;';
 		}
 		return $url;
 	}
@@ -1104,7 +1127,7 @@ class WebSvnConfig {
 				$url = substr($url, 0, -4);
 			}
 
-			if ($path && $path{0} != '/') {
+			if ($path && $path[0] != '/') {
 				$path = '/'.$path;
 			}
 
@@ -1115,7 +1138,7 @@ class WebSvnConfig {
 			if ($op == 'index') {
 				$url .= '/';
 			} else if (is_object($rep)) {
-				$url .= '/'.$rep->getDisplayName().str_replace('%2F', '/', rawurlencode($path));
+				$url .= '/'.$rep->getDisplayName().str_replace('%2F', '/', rawurlencode($path ?? ''));
 
 				if ($op && $op != 'dir' && $op != 'file') {
 					$params['op'] = $op;
@@ -1167,6 +1190,10 @@ class WebSvnConfig {
 				case 'comp':
 					$url = 'comp.php';
 					break;
+
+				case 'search':
+					$url = 'search.php';
+					break;
 			}
 
 			if (is_object($rep) && $op != 'index') {
@@ -1219,32 +1246,28 @@ class WebSvnConfig {
 	// Define directory path to use for --config-dir parameter
 	function setSvnConfigDir($path) {
 		$this->_svnConfigDir = $path;
-		$this->_updateSVNCommand();
+		$this->_updateSvnCommand();
 	}
 
 	// Define flag to use --trust-server-cert parameter
 	function setTrustServerCert() {
 		$this->_svnTrustServerCert = true;
-		$this->_updateSVNCommand();
+		$this->_updateSvnCommand();
 	}
 
 	// Define the location of the svn command (e.g. '/usr/bin')
 	function setSvnCommandPath($path) {
 		$this->_svnCommandPath = $path;
-		$this->_updateSVNCommand();
+		$this->_updateSvnCommand();
 	}
 
-	// Define a prefix to include before every SVN command (e.g. 'arch -i386')
-	function setSvnCommandPrefix($prefix) {
-		$this->_svnCommandPrefix = $prefix;
-		$this->_updateSVNCommand();
+	// Define the location of the svnauthz command (e.g. '/usr/bin')
+	function setSvnAuthzCommandPath($path) {
+		$this->_setPath($this->svnAuthz, $path, 'svnauthz', 'accessof');
 	}
 
-	function _updateSVNCommand() {
+	function _updateSvnCommand() {
 		$this->_setPath($this->svn, $this->_svnCommandPath, 'svn', '--non-interactive --config-dir '.$this->_svnConfigDir.($this->_svnTrustServerCert ? ' --trust-server-cert' : ''));
-		$this->_setPath($this->svnAuthz, $this->_svnCommandPath, 'svnauthz', 'accessof');
-		$this->svn = $this->_svnCommandPrefix.' '.$this->svn;
-		$this->svnAuthz = $this->_svnCommandPrefix.' '.$this->svnAuthz;
 	}
 
 	function getSvnCommand() {
@@ -1338,6 +1361,17 @@ class WebSvnConfig {
 		return $this->locale;
 	}
 
+	// setCharacterEncodings
+	//
+	// Set the array of character encodings for user files.
+	function setCharacterEncodings($characterEncodings) {
+		$this->characterEncodings = $characterEncodings;
+	}
+
+	function getCharacterEncodings() {
+		return $this->characterEncodings;
+	}
+
 	// setDefaultFileDlMode
 	//
 	// Define the default file download mode - one of [gzip, zip, plain]
@@ -1563,6 +1597,14 @@ class WebSvnConfig {
 		return $this->openTree;
 	}
 
+	function setLoadAllRepos($flag) {
+		$this->loadAllRepos = $flag;
+	}
+
+	function showLoadAllRepos() {
+		return $this->loadAllRepos;
+	}
+
 	function setAlphabeticOrder($flag) {
 		$this->alphabetic = $flag;
 	}
@@ -1657,4 +1699,16 @@ class WebSvnConfig {
 	}
 
 	// }}}
+
+	// {{{ Change the name of the breadcrumb root-phrase to that of the current repo?
+
+	function setBreadcrumbRepoRootAsRepo($newValue) {
+		$this->breadcrumbRepoRootAsRepo = $newValue;
+	}
+
+	function getBreadcrumbRepoRootAsRepo() {
+		return $this->breadcrumbRepoRootAsRepo;
+	}
+
+	// }}}
 }
