@@ -288,12 +288,10 @@ function listCharacterData($parser, $data) {
 
 		case 'DATE':
 			if ($debugxml) print 'Date: '.$data."\n";
-			$data = trim($data);
 			if ($data === false || $data === '') return;
-
 			$committime = parseSvnTimestamp($data);
 			$curList->curEntry->committime = $committime;
-			$curList->curEntry->date = strftime('%Y-%m-%d %H:%M:%S', $committime);
+			$curList->curEntry->date = date('Y-m-d H:i:s', $committime);
 			$curList->curEntry->age = datetimeFormatDuration(max(time() - $committime, 0), true, true);
 			break;
 	}
@@ -380,6 +378,34 @@ function logEndElement($parser, $name) {
 			break;
 
 		case 'PATH':
+			// The XML returned when a file is renamed/branched in inconsistent.
+			// In the case of a branch, the path doesn't include the leafname.
+			// In the case of a rename, it does.	Ludicrous.
+
+			if (!empty($curLog->path)) {
+				$pos = strrpos($curLog->path, '/');
+				$curpath = substr($curLog->path, 0, $pos);
+				$leafname = substr($curLog->path, $pos + 1);
+			} else {
+				$curpath = '';
+				$leafname = '';
+			}
+
+			$curMod = $curLog->curEntry->curMod;
+			if ($curMod->action == 'A') {
+				if ($debugxml) print 'Examining added path "'.$curMod->copyfrom.'" - Current path = "'.$curpath.'", leafname = "'.$leafname.'"'."\n";
+				if ($curMod->path == $curLog->path) {
+					// For directories and renames
+					$curLog->path = $curMod->copyfrom;
+				} else if ($curMod->path == $curpath || $curMod->path == $curpath.'/') {
+					// Logs of files that have moved due to branching
+					$curLog->path = $curMod->copyfrom.'/'.$leafname;
+				} else {
+					$curLog->path = str_replace($curMod->path, $curMod->copyfrom, $curLog->path);
+				}
+				if ($debugxml) print 'New path for comparison: "'.$curLog->path.'"'."\n";
+			}
+
 			if ($debugxml) print 'Ending path'."\n";
 			$curLog->curEntry->mods[] = $curLog->curEntry->curMod;
 			break;
@@ -411,12 +437,10 @@ function logCharacterData($parser, $data) {
 
 		case 'DATE':
 			if ($debugxml) print 'Date: '.$data."\n";
-			$data = trim($data);
 			if ($data === false || $data === '') return;
-
 			$committime = parseSvnTimestamp($data);
 			$curLog->curEntry->committime = $committime;
-			$curLog->curEntry->date = strftime('%Y-%m-%d %H:%M:%S', $committime);
+			$curLog->curEntry->date = date('Y-m-d H:i:s', $committime);
 			$curLog->curEntry->age = datetimeFormatDuration(max(time() - $committime, 0), true, true);
 			break;
 
@@ -429,38 +453,8 @@ function logCharacterData($parser, $data) {
 
 		case 'PATH':
 			if ($debugxml) print 'Path name: '.$data."\n";
-			$data = trim($data);
 			if ($data === false || $data === '') return;
-
 			$curLog->curEntry->curMod->path .= $data;
-
-			// The XML returned when a file is renamed/branched in inconsistent.
-			// In the case of a branch, the path doesn't include the leafname.
-			// In the case of a rename, it does.	Ludicrous.
-
-			if (!empty($curLog->path)) {
-				$pos = strrpos($curLog->path, '/');
-				$curpath = substr($curLog->path, 0, $pos);
-				$leafname = substr($curLog->path, $pos + 1);
-			} else {
-				$curpath = '';
-				$leafname = '';
-			}
-
-			$curMod = $curLog->curEntry->curMod;
-			if ($curMod->action == 'A') {
-				if ($debugxml) print 'Examining added path "'.$curMod->copyfrom.'" - Current path = "'.$curpath.'", leafname = "'.$leafname.'"'."\n";
-				if ($data == $curLog->path) {
-					// For directories and renames
-					$curLog->path = $curMod->copyfrom;
-				} else if ($data == $curpath || $data == $curpath.'/') {
-					// Logs of files that have moved due to branching
-					$curLog->path = $curMod->copyfrom.'/'.$leafname;
-				} else {
-					$curLog->path = str_replace($curMod->path, $curMod->copyfrom, $curLog->path);
-				}
-				if ($debugxml) print 'New path for comparison: "'.$curLog->path.'"'."\n";
-			}
 			break;
 	}
 }
@@ -475,7 +469,7 @@ function logCharacterData($parser, $data) {
 
 function _topLevel($entry) {
 	// To be at top level, there must be one space before the entry
-	return (strlen($entry) > 1 && $entry{0} == ' ' && $entry{1} != ' ');
+	return (strlen($entry) > 1 && $entry[0] == ' ' && $entry[ 1 ] != ' ');
 }
 
 // Function to sort two given directory entries.
@@ -486,8 +480,8 @@ function _listSort($e1, $e2) {
 
 	$file1 = $e1->file;
 	$file2 = $e2->file;
-	$isDir1 = ($file1{strlen($file1) - 1} == '/');
-	$isDir2 = ($file2{strlen($file2) - 1} == '/');
+	$isDir1 = ($file1[strlen($file1) - 1] == '/');
+	$isDir2 = ($file2[strlen($file2) - 1] == '/');
 
 	if (!$config->isAlphabeticOrder()) {
 		if ($isDir1 && !$isDir2) return -1;
@@ -517,7 +511,7 @@ function encodePath($uri) {
 	$parts = explode('/', $uri);
 	$partscount = count($parts);
 	for ($i = 0; $i < $partscount; $i++) {
-		// do not urlencode the 'svn+ssh://' part!
+		// do not rawurlencode the 'svn+ssh://' part!
 		if ($i != 0 || $parts[$i] != 'svn+ssh:') {
 			$parts[$i] = rawurlencode($parts[$i]);
 		}
@@ -550,7 +544,7 @@ function _equalPart($str1, $str2) {
 	$len2 = strlen($str2);
 	$i = 0;
 	while ($i < $len1 && $i < $len2) {
-		if (strcmp($str1{$i}, $str2{$i}) != 0) {
+		if (strcmp($str1[$i], $str2[$i]) != 0) {
 			break;
 		}
 		$i++;
@@ -641,12 +635,12 @@ class SVNRepository {
 
 		$filename = basename($path);
 		$ext = strrchr($path, '.');
-		
+
 		$lang = false;
 		if (array_key_exists($filename, $extEnscript)) {
 			$lang = $extEnscript[$filename];
-		} else if ($ext && array_key_exists($ext, $extEnscript)) {
-			$lang = $extEnscript[$ext];
+		} else if ($ext && array_key_exists(strtolower($ext), $extEnscript)) {
+			$lang = $extEnscript[strtolower($ext)];
 		}
 
 		$cmd = $config->enscript.' --language=html';
@@ -681,8 +675,13 @@ class SVNRepository {
 			$tempname = tempnamWithCheck($config->getTempDir(), '');
 		}
 		$highlighted = true;
+		$shouldTrimOutput = false;
+		$explodeStr = "\n";
 		if ($highlight != 'no' && $config->useGeshi && $geshiLang = $this->highlightLanguageUsingGeshi($path)) {
-			$this->applyGeshi($path, $tempname, $geshiLang, $rev, $peg);
+			$this->applyGeshi($path, $tempname, $geshiLang, $rev, $peg, false, $highlight);
+			// Geshi outputs in HTML format, enscript does not
+			$shouldTrimOutput = true;
+			$explodeStr = "<br />";
 		} else if ($highlight != 'no' && $config->useEnscript) {
 			// Get the files, feed it through enscript, then remove the enscript headers using sed
 			// Note that the sed command returns only the part of the file between <PRE> and </PRE>.
@@ -695,21 +694,16 @@ class SVNRepository {
 			$cmd = $this->svnCommandString('cat', $path, $rev, $peg);
 			$cmd = $cmd.' > '.quote($filename);
 		}
+
 		if (isset($cmd)) {
-			$descriptorspec = array(2 => array('pipe', 'w')); // stderr
-			$resource = proc_open($cmd, $descriptorspec, $pipes);
-			$error = '';
-			while (!feof($pipes[2])) {
-				$error .= fgets($pipes[2]);
-			}
-			$error = trim($error);
-			fclose($pipes[2]);
-			proc_close($resource);
+			$error	= '';
+			$output	= runCommand($cmd, true, $error);
 
 			if (!empty($error)) {
 				global $lang;
 				_logError($lang['BADCMD'].': '.$cmd);
 				_logError($error);
+
 				global $vars;
 				$vars['warning'] = nl2br(escape(toOutputEncoding($error)));
 			}
@@ -723,14 +717,17 @@ class SVNRepository {
 			$dst = fopen($filename, 'w');
 			if ($dst) {
 				$content = file_get_contents($tempname);
-				$content = explode('<br />', $content);
+				$content = explode($explodeStr, $content);
 
 				// $attributes is used to remember what highlighting attributes
 				// are in effect from one line to the next
 				$attributes = array(); // start with no attributes in effect
 
 				foreach ($content as $line) {
-					fputs($dst, $this->highlightLine(trim($line), $attributes)."\n");
+					if ($shouldTrimOutput) {
+						$line = trim($line);
+					}
+					fputs($dst, $this->highlightLine($line, $attributes)."\n");
 				}
 				fclose($dst);
 			}
@@ -748,6 +745,7 @@ class SVNRepository {
 	// check if geshi can highlight the given extension and return the language
 
 	function highlightLanguageUsingGeshi($path) {
+		global $config;
 		global $extGeshi;
 
 		$filename = basename($path);
@@ -755,9 +753,11 @@ class SVNRepository {
 		if (substr($ext, 0, 1) == '.') $ext = substr($ext, 1);
 
 		foreach ($extGeshi as $language => $extensions) {
-			if (in_array($filename, $extensions) || in_array($ext, $extensions)) {
+			if (in_array($filename, $extensions) || in_array(strtolower($ext), $extensions)) {
 				if ($this->geshi === null) {
-					require_once 'geshi.php';
+					if (!defined('USE_AUTOLOADER')) {
+						require_once $config->getGeshiScript();
+					}
 					$this->geshi = new GeSHi();
 				}
 				$this->geshi->set_language($language);
@@ -775,39 +775,45 @@ class SVNRepository {
 	//
 	// perform syntax highlighting using geshi
 
-	function applyGeshi($path, $filename, $language, $rev, $peg = '', $return = false) {
+	function applyGeshi($path, $filename, $language, $rev, $peg = '', $return = false, $highlight = 'file') {
+		global $config;
+
 		// Output the file to the filename
-		$cmd = $this->svnCommandString('cat', $path, $rev, $peg).' > '.quote($filename);
-		$descriptorspec = array(2 => array('pipe', 'w')); // stderr
-		$resource = proc_open($cmd, $descriptorspec, $pipes);
-		$error = '';
-		while (!feof($pipes[2])) {
-			$error .= fgets($pipes[2]);
-		}
-		$error = trim($error);
-		fclose($pipes[2]);
-		proc_close($resource);
+		$error	= '';
+		$cmd	= $this->svnCommandString('cat', $path, $rev, $peg).' > '.quote($filename);
+		$output	= runCommand($cmd, true, $error);
 
 		if (!empty($error)) {
 			global $lang;
 			_logError($lang['BADCMD'].': '.$cmd);
 			_logError($error);
+
 			global $vars;
 			$vars['warning'] = 'Unable to cat file: '.nl2br(escape(toOutputEncoding($error)));
 			return;
 		}
 
 		$source = file_get_contents($filename);
+
 		if ($this->geshi === null) {
-			require_once 'geshi.php';
+			if (!defined('USE_AUTOLOADER')) {
+				require_once $config->getGeshiScript();
+			}
 			$this->geshi = new GeSHi();
 		}
+
 		$this->geshi->set_source($source);
 		$this->geshi->set_language($language);
 		$this->geshi->set_header_type(GESHI_HEADER_NONE);
 		$this->geshi->set_overall_class('geshi');
 		$this->geshi->set_tab_width($this->repConfig->getExpandTabsBy());
 
+		if ($highlight == 'file') {
+			$this->geshi->enable_line_numbers(GESHI_FANCY_LINE_NUMBERS);
+			$this->geshi->set_overall_id('geshi');
+			$this->geshi->enable_ids(true);
+		}
+
 		if ($return) {
 			return $this->geshi->parse_code();
 		} else {
@@ -862,26 +868,64 @@ class SVNRepository {
 
 	// }}}
 
+	// {{{ listReadmeContents
+	//
+	// Parse the README.md file
+	function listReadmeContents($path, $rev = 0, $peg = '') {
+		global $config;
+
+		$file = "README.md";
+
+		if ($this->isFile($path.$file) != True)
+		{
+			return;
+		}
+
+		if (!$config->getUseParsedown())
+		{
+			return;
+		}
+
+		// Autoloader handles most of the time
+		if (!defined('USE_AUTOLOADER')) {
+			require_once $config->getParsedownScript();
+		}
+
+		$mdParser = new Parsedown();
+		$cmd = $this->svnCommandString('cat', $path.$file, $rev, $peg);
+
+		if (!($result = popenCommand($cmd, 'r')))
+		{
+			return;
+		}
+
+		echo('<div id="wrap">');
+		while (!feof($result))
+		{
+			$line = fgets($result, 1024);
+			echo $mdParser->text($line);
+		}
+		echo('</div>');
+		pclose($result);
+
+	}
+
+	// }}}
+
 	// {{{ getBlameDetails
 	//
 	// Dump the blame content of a file to the given filename
 
 	function getBlameDetails($path, $filename, $rev = 0, $peg = '') {
-		$cmd = $this->svnCommandString('blame', $path, $rev, $peg).' > '.quote($filename);
-		$descriptorspec = array(2 => array('pipe', 'w')); // stderr
-		$resource = proc_open($cmd, $descriptorspec, $pipes);
-		$error = '';
-		while (!feof($pipes[2])) {
-			$error .= fgets($pipes[2]);
-		}
-		$error = trim($error);
-		fclose($pipes[2]);
-		proc_close($resource);
+		$error	= '';
+		$cmd	= $this->svnCommandString('blame', $path, $rev, $peg).' > '.quote($filename);
+		$output	= runCommand($cmd, true, $error);
 
 		if (!empty($error)) {
 			global $lang;
 			_logError($lang['BADCMD'].': '.$cmd);
 			_logError($error);
+
 			global $vars;
 			$vars['warning'] = 'No blame info: '.nl2br(escape(toOutputEncoding($error)));
 		}
@@ -922,7 +966,7 @@ class SVNRepository {
 	// Exports the directory to the given location
 
 	function exportRepositoryPath($path, $filename, $rev = 0, $peg = '') {
-		$cmd = $this->svnCommandString('export', $path, $rev, $peg).' '.quote($filename);
+		$cmd = $this->svnCommandString('export', $path, $rev, $peg).' '.quote($filename.'@');
 		$retcode = 0;
 		execCommand($cmd, $retcode);
 		if ($retcode != 0) {
@@ -934,20 +978,74 @@ class SVNRepository {
 
 	// }}}
 
+	// {{{ _xmlParseCmdOutput
+
+	function _xmlParseCmdOutput($cmd, $startElem, $endElem, $charData) {
+		$error		= '';
+		$lines		= runCommand($cmd, false, $error);
+		$linesCnt	= count($lines);
+		$xml_parser	= xml_parser_create('UTF-8');
+
+		xml_parser_set_option($xml_parser, XML_OPTION_CASE_FOLDING, true);
+		xml_set_element_handler($xml_parser, $startElem, $endElem);
+		xml_set_character_data_handler($xml_parser, $charData);
+
+		for ($i = 0; $i < $linesCnt; ++$i) {
+			$line	= $lines[$i] . "\n";
+			$isLast	= $i == ($linesCnt - 1);
+
+			if (xml_parse($xml_parser, $line, $isLast)) {
+				continue;
+			}
+
+			$errorMsg = sprintf('XML error: %s (%d) at line %d column %d byte %d'."\n".'cmd: %s',
+								xml_error_string(xml_get_error_code($xml_parser)),
+								xml_get_error_code($xml_parser),
+								xml_get_current_line_number($xml_parser),
+								xml_get_current_column_number($xml_parser),
+								xml_get_current_byte_index($xml_parser),
+								$cmd);
+
+			if (xml_get_error_code($xml_parser) == 5) {
+				break;
+			}
+
+			// errors can contain sensitive info! don't echo this ~J
+			_logError($errorMsg);
+			exit;
+		}
+
+		xml_parser_free($xml_parser);
+		if (empty($error)) {
+			return;
+		}
+
+		$error = toOutputEncoding(nl2br(str_replace('svn: ', '', $error)));
+		global $lang;
+		_logError($lang['BADCMD'].': '.$cmd);
+		_logError($error);
+
+		global $vars;
+		if (strstr($error, 'found format')) {
+			$vars['error'] = 'Repository uses a newer format than Subversion '.$config->getSubversionVersion().' can read. ("'.nl2br(escape(toOutputEncoding(substr($error, strrpos($error, 'Expected'))))).'.")';
+		} else if (strstr($error, 'No such revision')) {
+			$vars['warning'] = 'Revision '.$rev.' of this resource does not exist.';
+		} else {
+			$vars['error'] = $lang['BADCMD'].': <code>'.escape(stripCredentialsFromCommand($cmd)).'</code><br />'.nl2br(escape(toOutputEncoding($error)));
+		}
+	}
+
+	// }}}
+
 	// {{{ getInfo
 
 	function getInfo($path, $rev = 0, $peg = '') {
 		global $config, $curInfo;
 
-		$xml_parser = xml_parser_create('UTF-8');
-		xml_parser_set_option($xml_parser, XML_OPTION_CASE_FOLDING, true);
-		xml_set_element_handler($xml_parser, 'infoStartElement', 'infoEndElement');
-		xml_set_character_data_handler($xml_parser, 'infoCharacterData');
-
 		// Since directories returned by svn log don't have trailing slashes (:-(), we need to remove
 		// the trailing slash from the path for comparison purposes
 
-		if ($path{strlen($path) - 1} == '/' && $path != '/') {
+		if ($path[strlen($path) - 1] == '/' && $path != '/') {
 			$path = substr($path, 0, -1);
 		}
 
@@ -962,67 +1060,7 @@ class SVNRepository {
 		}
 
 		$cmd = $this->svnCommandString('info --xml', $path, $rev, $peg);
-
-		$descriptorspec = array(0 => array('pipe', 'r'), 1 => array('pipe', 'w'), 2 => array('pipe', 'w'));
-
-		$resource = proc_open($cmd, $descriptorspec, $pipes);
-
-		if (!is_resource($resource)) {
-			global $lang;
-			echo $lang['BADCMD'].': <code>'.escape(stripCredentialsFromCommand($cmd)).'</code>';
-			exit;
-		}
-
-		$handle = $pipes[1];
-		$firstline = true;
-		while (!feof($handle)) {
-			$line = fgets($handle);
-			if (!xml_parse($xml_parser, $line, feof($handle))) {
-				$errorMsg = sprintf('XML error: %s (%d) at line %d column %d byte %d'."\n".'cmd: %s',
-									xml_error_string(xml_get_error_code($xml_parser)),
-									xml_get_error_code($xml_parser),
-									xml_get_current_line_number($xml_parser),
-									xml_get_current_column_number($xml_parser),
-									xml_get_current_byte_index($xml_parser),
-									$cmd);
-				if (xml_get_error_code($xml_parser) != 5) {
-					// errors can contain sensitive info! don't echo this ~J
-					_logError($errorMsg);
-					exit;
-				} else {
-					break;
-				}
-			}
-		}
-
-		$error = '';
-		while (!feof($pipes[2])) {
-			$error .= fgets($pipes[2]);
-		}
-		$error = toOutputEncoding(trim($error));
-
-		fclose($pipes[0]);
-		fclose($pipes[1]);
-		fclose($pipes[2]);
-
-		proc_close($resource);
-		xml_parser_free($xml_parser);
-
-		if (!empty($error)) {
-			$error = toOutputEncoding(nl2br(str_replace('svn: ', '', $error)));
-			global $lang;
-			_logError($lang['BADCMD'].': '.$cmd);
-			_logError($error);
-			global $vars;
-			if (strstr($error, 'found format')) {
-				$vars['error'] = 'Repository uses a newer format than Subversion '.$config->getSubversionVersion().' can read. ("'.nl2br(escape(toOutputEncoding(substr($error, strrpos($error, 'Expected'))))).'.")';
-			} else if (strstr($error, 'No such revision')) {
-				$vars['warning'] = 'Revision '.escape($rev).' of this resource does not exist.';
-			} else {
-				$vars['error'] = $lang['BADCMD'].': <code>'.escape(stripCredentialsFromCommand($cmd)).'</code><br />'.nl2br(escape(toOutputEncoding($error)));
-			}
-			return null;
-		}
+		$this->_xmlParseCmdOutput($cmd, 'infoStartElement', 'infoEndElement', 'infoCharacterData');
 
 		if ($this->repConfig->subpath !== null) {
 			if (substr($curInfo->path, 0, strlen($this->repConfig->subpath) + 1) === '/'. $this->repConfig->subpath) {
@@ -1043,15 +1081,10 @@ class SVNRepository {
 	function getList($path, $rev = 0, $peg = '') {
 		global $config, $curList;
 
-		$xml_parser = xml_parser_create('UTF-8');
-		xml_parser_set_option($xml_parser, XML_OPTION_CASE_FOLDING, true);
-		xml_set_element_handler($xml_parser, 'listStartElement', 'listEndElement');
-		xml_set_character_data_handler($xml_parser, 'listCharacterData');
-
 		// Since directories returned by svn log don't have trailing slashes (:-(), we need to remove
 		// the trailing slash from the path for comparison purposes
 
-		if ($path{strlen($path) - 1} == '/' && $path != '/') {
+		if ($path[strlen($path) - 1] == '/' && $path != '/') {
 			$path = substr($path, 0, -1);
 		}
 
@@ -1067,89 +1100,63 @@ class SVNRepository {
 				$rev = $headlog->entries[0]->rev;
 		}
 
-		$cmd = $this->svnCommandString('list --xml', $path, $rev, $peg);
-
-		$descriptorspec = array(0 => array('pipe', 'r'), 1 => array('pipe', 'w'), 2 => array('pipe', 'w'));
-
-		$resource = proc_open($cmd, $descriptorspec, $pipes);
-
-		if (!is_resource($resource)) {
-			global $lang;
-			echo $lang['BADCMD'].': <code>'.escape(stripCredentialsFromCommand($cmd)).'</code>';
-			exit;
+		if ($config->showLoadAllRepos()) {
+			$cmd = $this->svnCommandString('list -R --xml', $path, $rev, $peg);
+			$this->_xmlParseCmdOutput($cmd, 'listStartElement', 'listEndElement', 'listCharacterData');
+		}
+		else {
+			$cmd = $this->svnCommandString('list --xml', $path, $rev, $peg);
+			$this->_xmlParseCmdOutput($cmd, 'listStartElement', 'listEndElement', 'listCharacterData');
+			usort($curList->entries, '_listSort');
 		}
 
-		$handle = $pipes[1];
-		$firstline = true;
-		while (!feof($handle)) {
-			$line = fgets($handle);
-			if (!xml_parse($xml_parser, $line, feof($handle))) {
-				$errorMsg = sprintf('XML error: %s (%d) at line %d column %d byte %d'."\n".'cmd: %s',
-									xml_error_string(xml_get_error_code($xml_parser)),
-									xml_get_error_code($xml_parser),
-									xml_get_current_line_number($xml_parser),
-									xml_get_current_column_number($xml_parser),
-									xml_get_current_byte_index($xml_parser),
-									$cmd);
-				if (xml_get_error_code($xml_parser) != 5) {
-					// errors can contain sensitive info! don't echo this ~J
-					_logError($errorMsg);
-					exit;
-				} else {
-					break;
-				}
-			}
-		}
-
-		$error = '';
-		while (!feof($pipes[2])) {
-			$error .= fgets($pipes[2]);
-		}
-		$error = toOutputEncoding(trim($error));
-
-		fclose($pipes[0]);
-		fclose($pipes[1]);
-		fclose($pipes[2]);
-
-		proc_close($resource);
-		xml_parser_free($xml_parser);
-
-		if (!empty($error)) {
-			$error = toOutputEncoding(nl2br(str_replace('svn: ', '', $error)));
-			global $lang;
-			_logError($lang['BADCMD'].': '.$cmd);
-			_logError($error);
-			global $vars;
-			if (strstr($error, 'found format')) {
-				$vars['error'] = 'Repository uses a newer format than Subversion '.$config->getSubversionVersion().' can read. ("'.nl2br(escape(toOutputEncoding(substr($error, strrpos($error, 'Expected'))))).'.")';
-			} else if (strstr($error, 'No such revision')) {
-				$vars['warning'] = 'Revision '.escape($rev).' of this resource does not exist.';
-			} else {
-				$vars['error'] = $lang['BADCMD'].': <code>'.escape(stripCredentialsFromCommand($cmd)).'</code><br />'.nl2br(escape(toOutputEncoding($error)));
-			}
-			return null;
-		}
-
-		// Sort the entries into alphabetical order
-		usort($curList->entries, '_listSort');
 		return $curList;
 	}
 
 	// }}}
 
+	// {{{ getListSearch
+
+	function getListSearch($path, $term = '', $rev = 0, $peg = '') {
+		global $config, $curList;
+
+		// Since directories returned by "svn log" don't have trailing slashes (:-(), we need to
+		// remove the trailing slash from the path for comparison purposes.
+		if (($path[strlen($path) - 1] == '/') && ($path != '/')) {
+			$path = substr($path, 0, -1);
+		}
+
+		$curList			= new SVNList;
+		$curList->entries	= array();
+		$curList->path		= $path;
+
+		// Get the list info
+
+		if ($rev == 0) {
+			$headlog = $this->getLog('/', '', '', true, 1);
+			if ($headlog && isset($headlog->entries[0]))
+				$rev = $headlog->entries[0]->rev;
+		}
+
+		$term	= escapeshellarg($term);
+		$cmd	= 'list -R --search ' . $term . ' --xml';
+		$cmd	= $this->svnCommandString($cmd, $path, $rev, $peg);
+		$this->_xmlParseCmdOutput($cmd, 'listStartElement', 'listEndElement', 'listCharacterData');
+
+		return $curList;
+	}
+
+	// }}}
+
+
 	// {{{ getLog
 
 	function getLog($path, $brev = '', $erev = 1, $quiet = false, $limit = 2, $peg = '', $verbose = false) {
 		global $config, $curLog;
 
-		$xml_parser = xml_parser_create('UTF-8');
-		xml_parser_set_option($xml_parser, XML_OPTION_CASE_FOLDING, true);
-		xml_set_element_handler($xml_parser, 'logStartElement', 'logEndElement');
-		xml_set_character_data_handler($xml_parser, 'logCharacterData');
-
 		// Since directories returned by svn log don't have trailing slashes (:-(),
 		// we must remove the trailing slash from the path for comparison purposes.
-		if ($path != '/' && $path{strlen($path) - 1} == '/') {
+		if (!empty($path) && $path != '/' && $path[strlen($path) - 1] == '/') {
 			$path = substr($path, 0, -1);
 		}
 
@@ -1158,70 +1165,11 @@ class SVNRepository {
 		$curLog->path = $path;
 
 		// Get the log info
-		$effectiveRev = ($brev && $erev ? $brev.':'.$erev : ($brev ? $brev.':1' : ''));
-		$effectivePeg = ($peg ? $peg : ($brev ? $brev : ''));
-		$cmd = $this->svnCommandString('log --xml '.($verbose ? '--verbose' : ($quiet ? '--quiet' : '')).($limit != 0 ? ' --limit '.$limit : ''), $path, $effectiveRev, $effectivePeg);
+		$effectiveRev	= ($brev && $erev ? $brev.':'.$erev : ($brev ? $brev.':1' : ''));
+		$effectivePeg	= ($peg ? $peg : ($brev ? $brev : ''));
+		$cmd			= $this->svnCommandString('log --xml '.($verbose ? '--verbose' : ($quiet ? '--quiet' : '')).($limit != 0 ? ' --limit '.$limit : ''), $path, $effectiveRev, $effectivePeg);
 
-		$descriptorspec = array(0 => array('pipe', 'r'), 1 => array('pipe', 'w'), 2 => array('pipe', 'w'));
-
-		$resource = proc_open($cmd, $descriptorspec, $pipes);
-
-		if (!is_resource($resource)) {
-			global $lang;
-			echo $lang['BADCMD'].': <code>'.escape(stripCredentialsFromCommand($cmd)).'</code>';
-			exit;
-		}
-
-		$handle = $pipes[1];
-		$firstline = true;
-		while (!feof($handle)) {
-			$line = fgets($handle);
-			if (!xml_parse($xml_parser, $line, feof($handle))) {
-				$errorMsg = sprintf('XML error: %s (%d) at line %d column %d byte %d'."\n".'cmd: %s',
-									xml_error_string(xml_get_error_code($xml_parser)),
-									xml_get_error_code($xml_parser),
-									xml_get_current_line_number($xml_parser),
-									xml_get_current_column_number($xml_parser),
-									xml_get_current_byte_index($xml_parser),
-									$cmd);
-				if (xml_get_error_code($xml_parser) != 5) {
-					// errors can contain sensitive info! don't echo this ~J
-					_logError($errorMsg);
-					exit;
-				} else {
-					break;
-				}
-			}
-		}
-
-		$error = '';
-		while (!feof($pipes[2])) {
-			$error .= fgets($pipes[2]);
-		}
-		$error = trim($error);
-
-		fclose($pipes[0]);
-		fclose($pipes[1]);
-		fclose($pipes[2]);
-
-		proc_close($resource);
-
-		if (!empty($error)) {
-			global $lang;
-			_logError($lang['BADCMD'].': '.$cmd);
-			_logError($error);
-			global $vars;
-			if (strstr($error, 'found format')) {
-				$vars['error'] = 'Repository uses a newer format than Subversion '.$config->getSubversionVersion().' can read. ("'.nl2br(escape(toOutputEncoding(substr($error, strrpos($error, 'Expected'))))).'.")';
-			} else if (strstr($error, 'No such revision')) {
-				$vars['warning'] = 'Revision '.escape($brev).' of this resource does not exist.';
-			} else {
-				$vars['error'] = $lang['BADCMD'].': <code>'.escape(stripCredentialsFromCommand($cmd)).'</code><br />'.nl2br(escape(toOutputEncoding($error)));
-			}
-			return null;
-		}
-
-		xml_parser_free($xml_parser);
+		$this->_xmlParseCmdOutput($cmd, 'logStartElement', 'logEndElement', 'logCharacterData');
 
 		foreach ($curLog->entries as $entryKey => $entry) {
 			$fullModAccess = true;
